/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.vii.brillien.ignition.transport;

import com.vii.brillien.ignition.BrillienStatics;
import com.vii.streamline.services.MathServices;

/**
 * Utility class providing static data and services for all transport-layer-related activity.
 */
public class TransportStatics {

    /**
     * Mime-type for the transport-layer
     */
    public static final String              TRANSPORT_MIME_TYPE             = "application/brillien";

    public static final int                 POLLING_INTERVAL                = 5000;
    /**
     * Priority levels for messages
     */
    public static final int                 MESSAGE_PRIORITY_LEVEL_1        = 1;
    public static final int                 MESSAGE_PRIORITY_LEVEL_2        = 2;
    public static final int                 MESSAGE_PRIORITY_LEVEL_3        = 3;
    public static final int                 MESSAGE_PRIORITY_LEVEL_4        = 4;
    public static final int                 MESSAGE_PRIORITY_LEVEL_5        = 5;
    public static final int                 MESSAGE_PRIORITY_LEVEL_6        = 6;
    public static final int                 MESSAGE_PRIORITY_LEVEL_7        = 7;
    public static final int                 MESSAGE_PRIORITY_LEVEL_8        = 8;
    public static final int                 MESSAGE_PRIORITY_LEVEL_9        = 9;

    /**
     * Constants for transport providers to use persistency or not
     */
    public static final int                 MESSAGE_DELIVERY_NON_PERSISTENT = 1;
    public static final int                 MESSAGE_DELIVERY_PERSISTENT     = 2;

    /**
     * Generates new unique ID for a message
     */
    public static String newMessageId(){
        long nanoValue = ((long)(MathServices.random.nextDouble() * System.nanoTime()));
        return  BrillienStatics.SYSTEM_ID + "_" +
                System.currentTimeMillis() + "_" +
                nanoValue;
    }

}
